package com.example.myapplication;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.text.TextUtils;
import android.util.Log;
import android.widget.TextView;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.URL;
import java.security.GeneralSecurityException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import mlab.android.cloudvrconnect.IntentCVR;


public class MainActivity extends Activity {
    private static final String GLOBAL_DOMAIN = "cvr.myhuaweicloud.com";
    private final static int TYPE_ACCOUNT = 0;  //账号密码的连接方式
    private final static int TYPE_IAMTOKEN = 1; //IAMToken的连接方式
    private final static int TYPE_GPUINFO = 2;  //GPUIP + GPUToken的连接方式
    private final static String TAG = "OUTSIDE_ACTIVITY";
    protected static final int REQUEST_CODE = 1;
    private final static int MSG_HINT_UPDATE = 1000;

    private List<String> mPermissionList = new ArrayList<>();
    public static String[] permissions = {
        Manifest.permission.READ_EXTERNAL_STORAGE,
        Manifest.permission.WRITE_EXTERNAL_STORAGE
    };
    Context context;
    private TextView tvHint;
    private Handler handler = new Handler() {
        @Override
        public void handleMessage(@NonNull Message msg) {
            if (msg.what == MSG_HINT_UPDATE) {
                tvHint.setText((String)msg.obj);
            }
            super.handleMessage(msg);
        }
    };

    private String deviceID;
    private String brand;
    private GpuInfo gpuInfo = null;
    private String iamToken = null;
    private String projectID;
    private boolean bJumpByIntent = false;
    //获取默认port值,非沙箱模式，请勿自行修改
    private int port = IntentCVR.getCvrParameters().getPort();

    //连接类型和连接所需的参数，这里是你注意和修改的参数<<<<<<<<<<<
    private int linkType = TYPE_ACCOUNT;
    private String appID = "appID";
    private String username = "username";
    private String userdomain = "userdomain";
    private String userPassword = "********";
    private String projectName = "projectName";
    private String serviceDomain = "serviceDomain";
    private String[] location = new String[]{};
    private String iamDomain = "iam.myhuaweicloud.com";
    private String gpuIpType = "";
    //连接类型和连接所需的参数，这里是你注意和修改的参数>>>>>>>>>>>

    @Override
    public void onCreate(Bundle savedInstanceState) {
        Log.i(TAG, "onCreate");
        setContentView(R.layout.activity_main);
        tvHint = findViewById(R.id.tv_hint);
        bJumpByIntent = false;
        context = this;
        IntentCVR.setErrCallback(new IntentCVR.CvrErrCallback() {
            @Override
            public void onErrMessage(String s, String s1) {
                //s为简略的错误信息，s1为详细的，当前会反馈的信息包含天空盒的错误信息和获取deviceID的错误信息
                Log.i(TAG,s + s1);
            }

            @Override
            public void onTokenOutdated() {
                //当前的token在后台的时间已经超出timelimit，重新建立连接，timelimit默认为60
                Log.i(TAG, "onTokenOutdated");
                link();
            }
        });
        start();
        super.onCreate(savedInstanceState);
    }

    private void start() {
        if (linkType == TYPE_GPUINFO) {
            //GPUINFO的连接方式需要获取相关的存储权限，用于获取deviceID
            if (isHasPermission()) {
                link();
            } else {
                requestPer();
            }
        } else {
            link();
        }
    }

    private void link() {
        Log.i(TAG, "link()");
        brand = IntentCVR.getCvrParameters().getBrand();
        deviceID = IntentCVR.getCvrParameters().getDeviceID(this);
        if (linkType == TYPE_ACCOUNT) {
            IntentCVR intentCVR = new IntentCVR(context);
            IntentCVR.CVRParameter customParameter;
            customParameter = new IntentCVR.CVRParameter(username, userdomain,
                    userPassword, projectName);
            customParameter.setLocation(location);
            customParameter.setServiceDomain(serviceDomain);
            customParameter.setAppId(appID);
            customParameter.setAppName(getString(R.string.app_name));
            customParameter.setPort(port);
            customParameter.setIamDomain(iamDomain);
            customParameter.setGpuIpType(gpuIpType);
            bJumpByIntent = true;
            String result = intentCVR.initiateLoad(customParameter, VrMainActivity.class);
            Log.i(TAG, "Init result : " + result);
            if (!result.equals("Succeed")) {
                bJumpByIntent = false;
            }
            return;
        }
        linkPlatform(new CallBack() {
            @Override
            public void startVR() {
                IntentCVR intentCVR = new IntentCVR(context);
                IntentCVR.CVRParameter customParameter;
                if (linkType == TYPE_IAMTOKEN && iamToken != null) {
                    customParameter = new IntentCVR.CVRParameter(iamToken);
                    customParameter.setProjectID(projectID);
                    customParameter.setServiceDomain(serviceDomain);
                    customParameter.setAppId(appID);
                    customParameter.setAppName(getString(R.string.app_name));
                    customParameter.setPort(port);
                    customParameter.setLocation(location);
                    customParameter.setIamDomain(iamDomain);
                    customParameter.setGpuIpType(gpuIpType);
                    bJumpByIntent = true;
                    String result = intentCVR.initiateLoad(customParameter, VrMainActivity.class);
                    if (!result.equals("Succeed")) {
                        bJumpByIntent = false;
                    }
                    Log.i(TAG, "Init result : " + result);
                }
                if (gpuInfo != null && linkType == TYPE_GPUINFO) {
                    customParameter = new IntentCVR.CVRParameter(gpuInfo.ip, gpuInfo.token);
                    customParameter.setServiceDomain(serviceDomain);
                    customParameter.setAppId(appID);
                    customParameter.setAppName(getString(R.string.app_name));
                    customParameter.setPort(port);
                    customParameter.setLocation(location);
                    customParameter.setIamDomain(iamDomain);
                    customParameter.setGpuIpType(gpuIpType);
                    bJumpByIntent = true;
                    String result = intentCVR.initiateLoad(customParameter, VrMainActivity.class);
                    if (!result.equals("Succeed")) {
                        bJumpByIntent = false;
                    }
                    Log.i(TAG, "Init result : " + result);
                }

            }
        });
    }

    @Override
    protected void onPause() {
        Log.i(TAG, "onPause()");
        if (!bJumpByIntent) {
            //此处的代码是为了在手柄点击home键中断连接操作后，对当前应用予以结束指令。
            //因为部分头显，如Oculus quest，在此Activity的逻辑运行过程中点击home键，再点击退出，通过日志发现无法被正常结束程序
            //Oculus quest2，在此Activity的逻辑运行过程中点击home键，退到主页后发现未出现应用是否结束的对话框，但通过日志发现仍运行在后台
            //若发现除Oculus之外的头显有类似现象，请及时联系我们
            //此处只是一个简单的处理范例——强行退出，可以根据自身需要对此处的逻辑进行修改，如应用打开再次回到前台时，重新访问调度
            System.exit(0);
        } else {
            bJumpByIntent = false;
        }
        super.onPause();
    }

    //检查是否已经有所需权限
    private boolean isHasPermission() {
        for (int i = 0; i < permissions.length; i++) {
            if (ActivityCompat.checkSelfPermission(this, permissions[i]) != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
        }
        return true;
    }

    //请求文件读写权限的结果
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        Log.i(TAG, "onRequestPermissionsResult");
        if (requestCode == REQUEST_CODE) {
            if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                link();
            }
        }
    }

    //请求缺失权限
    protected void requestPer() {
        mPermissionList.clear();
        for (int i = 0; i < permissions.length; i++) {
            if (ActivityCompat.checkSelfPermission(this, permissions[i]) != PackageManager.PERMISSION_GRANTED) {
                mPermissionList.add(permissions[i]);
            }
        }
        if (!mPermissionList.isEmpty()) {
            String[] permissions = mPermissionList.toArray(new String[mPermissionList.size()]);
            bJumpByIntent = true;
            ActivityCompat.requestPermissions(this, permissions, REQUEST_CODE);
        }
    }

    protected static SSLSocketFactory getDefaultSSLSocketFactory() {
        try {
            //获取默认ssl证书
            SSLContext sslContext = SSLContext.getInstance("TLS");
            // TrustManager信任所有的证书
            sslContext.init(null, new TrustManager[]{
                new X509TrustManager() {
                    public void checkClientTrusted(X509Certificate[] x509Certificates, String s) throws CertificateException {

                    }

                    public void checkServerTrusted(X509Certificate[] x509Certificates, String s) throws CertificateException {
                    }

                    public X509Certificate[] getAcceptedIssuers() {
                        return new X509Certificate[0];
                    }
                }
            }, null);
            return sslContext.getSocketFactory();
        } catch (GeneralSecurityException e) {
            throw new AssertionError();
        }
    }
    private static HostnameVerifier getHostnameVerifier() {
        return new HostnameVerifier() {
            @Override
            public boolean verify(String hostname, SSLSession session) {
                return true;
            }
        };
    }

    public void linkPlatform(final CallBack callBack) {
        Log.i(TAG, "linkPlatform()");
        new Thread(new Runnable() {
            @Override
            public void run() {
                Looper.prepare();
                getToken();
                if (TextUtils.isEmpty(iamToken) || TextUtils.isEmpty(projectID)) {
                    Log.i(TAG, "get token error");
                    return;
                }
                Log.i(TAG, "after get token");
                if (linkType == TYPE_IAMTOKEN) {
                    callBack.startVR();
                    return;
                }
                getGpuInfo(callBack);
                Looper.loop();
            }
        }).start();
    }

    private void getToken() {
        HttpsURLConnection connection = null;
        try {
            URL url = new URL("https://" + iamDomain + "/v3/auth/tokens?nocatalog=true&noroles=true");
            connection = (HttpsURLConnection) url.openConnection();
            connection.setInstanceFollowRedirects(false);
            connection.setRequestMethod("POST");
            connection.setRequestProperty("Charset", "UTF-8");
            connection.setRequestProperty("accept", "application/json");
            connection.setRequestProperty("Content-Type", "application/json");
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(60000);
            if (!iamDomain.equals(IntentCVR.DOMAIN_NORMAL)) {
                connection.setSSLSocketFactory(getDefaultSSLSocketFactory());
                connection.setHostnameVerifier(getHostnameVerifier());
            }
            connection.setDoOutput(true);
            connection.setDoInput(true);
            connection.setUseCaches(false);
            connection.connect();
            String parameterData = "";
            try {
                JSONObject parameterObject = new JSONObject();
                JSONObject authObject = new JSONObject();
                JSONObject identityObject = new JSONObject();
                JSONObject scopeObject = new JSONObject();
                JSONObject projectObject = new JSONObject();
                JSONObject passWordObject = new JSONObject();
                JSONObject userObject = new JSONObject();
                JSONObject domainObject = new JSONObject();

                parameterObject.put("auth", authObject);
                authObject.put("identity", identityObject);
                authObject.put("scope", scopeObject);
                JSONArray jsonArray = new JSONArray();
                jsonArray.put("password");
                identityObject.put("methods",jsonArray);
                identityObject.put("password",passWordObject);
                passWordObject.put("user",userObject);
                userObject.put("name",username);
                userObject.put("password",userPassword);
                userObject.put("domain",domainObject);
                domainObject.put("name",userdomain);
                scopeObject.put("project",projectObject);
                projectObject.put("name",projectName);
                parameterData = parameterObject.toString();
            } catch (JSONException e) {
                Log.e(TAG, e.toString());
            }
            Writer writer = new OutputStreamWriter(connection.getOutputStream());
            writer.write(parameterData);
            writer.flush();
            writer.close();

            String bodyString;
            int responseCode = connection.getResponseCode();
            if (responseCode == 200 || responseCode == 201) {
                Map<String, List<String>> map = connection.getHeaderFields();
                InputStream inputStream = connection.getInputStream();
                iamToken = map.get("X-Subject-Token").get(0);
                Log.d(TAG, "getToken: ContentLength" + connection.getContentLength());
                bodyString = readBody(inputStream);
                projectID = jsonAnalysProjectId(bodyString);
            } else {
                Log.i(TAG, "responseCode :" + responseCode);
                InputStream is = connection.getErrorStream();
                if (is != null) {
                    String bodyInput = readBody(is);
                    Log.i(TAG, "getToken failed: " + bodyInput);
                    JSONObject object = new JSONObject(bodyInput);
                    JSONObject error = (JSONObject) object.get("error");
                    String code = error.getString("code");
                    String message = error.getString("message");
                    Message msg = new Message();
                    msg.what = MSG_HINT_UPDATE;
                    msg.obj = "code:" + code + " message:" + message;
                    handler.sendMessage(msg);
                }
            }
        } catch (Exception e) {
            Log.e(TAG, e.toString());
            iamToken = "";
            projectID = "";
        }
    }

    public void getGpuInfo(CallBack callBack) {
        HttpsURLConnection connection = null;
        try {
            URL url;
            if (GLOBAL_DOMAIN.equals(serviceDomain)) {
                url = new URL("https://" + serviceDomain + "/v1/cvr/nodes/scheduler");
            } else {
                url = new URL("https://" + serviceDomain + "/v1/" + projectID + "/cvr/node/scheduler");
            }
            Log.i(TAG, url.toString());
            connection = (HttpsURLConnection) url.openConnection();
            connection.setInstanceFollowRedirects(false);
            connection.setRequestMethod("POST");
            connection.setRequestProperty("Charset", "UTF-8");
            connection.setRequestProperty("X-AUTH-TOKEN", iamToken);
            connection.setRequestProperty("accept", "application/json");
            connection.setRequestProperty("Content-Type", "application/json");
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(60000);
            if (!iamDomain.equals(IntentCVR.DOMAIN_NORMAL)) {
                connection.setSSLSocketFactory(getDefaultSSLSocketFactory());
                connection.setHostnameVerifier(getHostnameVerifier());
            }
            connection.setDoOutput(true);
            connection.setDoInput(true);
            connection.setUseCaches(false);
            connection.connect();
            String parameterData = "";
            try {
                JSONObject parameterObject = new JSONObject();
                parameterObject.put("app_id", appID);
                parameterObject.put("device_id", deviceID);
                parameterObject.put("device_brand", brand);
                parameterObject.put("location", getLocationArray(location));
                parameterObject.put("sandbox_enable", true);
                parameterObject.put("protocol_type", IntentCVR.getCvrParameters().getProtocolType());
                parameterObject.put("device_type", IntentCVR.getCvrParameters().getDeviceType());
                if (!TextUtils.isEmpty(gpuIpType)) {
                    parameterObject.put("gpu_ip_type", gpuIpType);
                }
                parameterData = parameterObject.toString();
            } catch (JSONException e) {
            }
            Writer writer = new OutputStreamWriter(connection.getOutputStream());
            writer.write(parameterData);
            writer.flush();
            writer.close();
            int responseCode = connection.getResponseCode();
            Log.i(TAG, "responseCode: " + responseCode);
            if (responseCode == 200) {
                InputStream is = connection.getInputStream();
                String bodyInput = readBody(is);
                gpuInfo = jsonAnalysToken(bodyInput);
                if (gpuInfo.port != null) {
                    port = Integer.parseInt(gpuInfo.port);
                }
                Log.i(TAG, "gpuInfo: " + gpuInfo.toString());
                callBack.startVR();
            } else {
                InputStream is = connection.getErrorStream();
                String bodyInput = readBody(is);
                JSONObject object = new JSONObject(bodyInput);
                String code = object.getString("error_code");
                String message = object.getString("error_msg");
                Log.i(TAG, "getGpuInfo error: " + bodyInput);
                Message msg = new Message();
                msg.what = MSG_HINT_UPDATE;
                msg.obj = "code:" + code + " message:" + message;
                handler.sendMessage(msg);
            }
        } catch (Exception e) {
            Log.e(TAG, e.toString());
        }
    }

    private static String readBody(InputStream inputStream) {
        InputStreamReader inputStreamReader = new InputStreamReader(inputStream);
        BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
        StringBuilder stringBuilder = new StringBuilder();

        String line;
        try {
            while ((line = bufferedReader.readLine()) != null) {
                stringBuilder.append(line);
            }
            bufferedReader.close();
            inputStreamReader.close();
        } catch (IOException e) {
        }
        return stringBuilder.toString();
    }

    private static String jsonAnalysProjectId(String body) {
        String s1 = body.replaceAll("\\{", "\\\n\\{");
        JSONObject object = null;
        try {
            object = new JSONObject(body);
            JSONObject token = object.getJSONObject("token");
            JSONObject project = token.getJSONObject("project");
            Object id = project.get("id");
            String s = id.toString();
            return s;
        } catch (JSONException e) {
        }
        return "";
    }

    private static GpuInfo jsonAnalysToken(String bodyInput) {
        GpuInfo gpuInfo = new GpuInfo();
        JSONObject object = null;
        try {
            object = new JSONObject(bodyInput);
            gpuInfo.ip = object.getString("gpu_ip");
            gpuInfo.token = object.getString("link_token");
            if (object.has("protocol_type")) {
                gpuInfo.protocolType = object.getString("protocol_type");
                gpuInfo.protocolSecret = object.getString("protocol_secret");
            } else {
                gpuInfo.protocolType = "udp";
                gpuInfo.protocolSecret = "";
            }
            if (object.has("port")) {
                gpuInfo.port = object.getString("port");
            }
        } catch (JSONException e) {
        }
        return gpuInfo;
    }

    private static JSONArray getLocationArray(String[] location) throws JSONException {
        if (location.length == 0) {
            return new JSONArray("[]");
        }
        StringBuilder builder = new StringBuilder();
        builder.append("[\"");
        for (String str : location) {
            builder.append(str);
            builder.append("\",\"");
        }
        builder.delete(builder.length() - 3, builder.length() - 1);
        builder.append("]");
        return new JSONArray(builder.toString());
    }

    public interface CallBack {
        void startVR();
    }

    public static class GpuInfo {
        public String ip;
        public String token;
        public String protocolType;
        public String protocolSecret;
        public String port;
    }
}