package com.example.myapplication;

import android.Manifest;
import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.ActivityCompat;
import android.view.KeyEvent;
import android.view.SurfaceView;
import android.view.Window;
import android.view.WindowManager;

import java.util.ArrayList;
import java.util.List;

import mlab.android.cloudvrconnect.CvrSDK;
import mlab.android.cloudvrconnect.util.Log;

/**
 * 抽象类，定义公用的方法
 * */
public class VrMainActivity extends Activity implements ActivityCompat.OnRequestPermissionsResultCallback {
    private static final String TAG = "VrMainActivity";
    protected static final int REQUEST_CODE = 1;

    private List<String> mPermissionList = new ArrayList<>();
    public static String[] permissions = {
        Manifest.permission.READ_EXTERNAL_STORAGE,
        Manifest.permission.WRITE_EXTERNAL_STORAGE
    };

    //若使用的头显为pico类型的，此处的变量名不能修改、混淆（pico官方native sdk内对此变量名有引用），否则会出现黑屏现象
    private SurfaceView MySurfaceView;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);// 设置全屏
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        setContentView(R.layout.activity_vr);
        MySurfaceView = findViewById(R.id.surfaceview);

        if (isHasPermission()) {
            CvrSDK.getInstance().onActivityCreate(this, MySurfaceView);
        } else {
            requestPer();
        }
        Log.d(TAG, "onCreate: finish request permission");
        super.onCreate(savedInstanceState);
    }

    @Override
    protected void onResume() {
        super.onResume();
        CvrSDK.getInstance().onActivityResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        CvrSDK.getInstance().onActivityPause();
    }

    @Override
    protected void onStop() {
        super.onStop();
        CvrSDK.getInstance().onActivityStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        CvrSDK.getInstance().onActivityDestroy();
    }

    //检查是否已经有所需权限
    private boolean isHasPermission() {
        for (int i = 0; i < permissions.length; i++) {
            if (ActivityCompat.checkSelfPermission(this, permissions[i]) != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
        }
        return true;
    }

    //请求缺失权限
    private void requestPer() {
        mPermissionList.clear();
        for (int i = 0; i < permissions.length; i++) {
            if (ActivityCompat.checkSelfPermission(this, permissions[i]) != PackageManager.PERMISSION_GRANTED) {
                mPermissionList.add(permissions[i]);
            }
        }
        if (!mPermissionList.isEmpty()) {
            String[] permissions = mPermissionList.toArray(new String[mPermissionList.size()]);//将List转为数组
            ActivityCompat.requestPermissions(this, permissions, REQUEST_CODE);
        }
    }

    /**
     * 请求文件读写权限的结果
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == REQUEST_CODE) {
            if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                CvrSDK.getInstance().onActivityCreate(this, MySurfaceView);
            } else {
                finish();
            }
        }
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        if (event.getAction() == KeyEvent.ACTION_DOWN || event.getAction() == KeyEvent.ACTION_UP) {
            return CvrSDK.getInstance().handleKeyEvent(event);
        } else {
            return super.dispatchKeyEvent(event);
        }
    }
}
